classdef Converter < handle
    
    % Direct access to the properties allowed
    properties ( Access = public )
        
        % Temperatures in Celsius and Fahrenheit
        tC, tF
        
    end
    
    % Define events that can be triggered: access is public
    events ( ListenAccess = public, NotifyAccess = public )
        
        changedTC
        changedTF
        
    end
    
    methods
        
        % Constructor
        function obj = Converter( )
            
            % Set listeners
            addlistener( obj, 'changedTC', @convertCtoF );
            addlistener( obj, 'changedTF', @convertFtoC );
            
            % Set reasonable starting values
            % This will trigger the changedTC event and update tF
            obj.tC = 0;
        end
        
        % Setters
        % The events will be triggered only if the properties change,
        % this prevents a cascade of events being generated
        function set.tC( obj, value )
            obj.tC = value;
            obj.notify( 'changedTC' );
        end
        
        function set.tF( obj, value )
            obj.tF = value;
            obj.notify( 'changedTF' );
        end
        
        function display( obj )
            fprintf( 1, ...
                [ 'Current temperature: %.2f Celsius corresponding ', ...
                'to %.2f Fahrenheit.\n' ], obj.tC, obj.tF );
        end
        
    end
    
    methods ( Access = private )
        
        function convertCtoF( obj, evnt )
            obj.tF = ( obj.tC * 9 / 5 ) + 32;
        end
        
        function convertFtoC( obj, evnt )
            obj.tC = ( obj.tF - 32 ) * 5 / 9;
        end
        
    end
    
end
